import re
from typing import List, Tuple

from pydantic import BaseModel


class SlugifyOptions(BaseModel):
    """
    Slugify options

    collapse_whitespace: Collapse the whitespaces. Default `True`.
    disallowed_characters: A regex for the disallowed_characters.
    separator: The separator. Default `-`.
    replacements: A list of replacement.
    """

    collapse_whitespace: bool = True
    disallowed_characters: str = r"[^\w\s-]"
    separator: str = "-"
    replacements: List[Tuple[str, str]] = []


def slugify(text: str, options: SlugifyOptions = SlugifyOptions()):
    """
    Slugify a text.

    text: The initial text.
    options: A `SlugifyOptions` object containing all the settings.
    return: The slugify text
    """
    for replacement in options.replacements:
        text = text.replace(replacement[0], replacement[1])

    text = re.sub(options.disallowed_characters, " ", text.lower())

    pattern = re.compile(r"\W+") if options.collapse_whitespace else re.compile(r"\W")
    return re.sub(pattern, options.separator, text).lower().strip("-_")



from litestar_utils.text import SlugifyOptions, slugify
from litestar_utils.timing import create_timing_middleware

__author__ = "Alessandro Ferrini"

__email__ = "im@alessandroferrini.com"

credits = [
    "Alessandro Ferrini",
]

__all__ = ["create_timing_middleware", "slugify", "SlugifyOptions"]



from time import time
from typing import TYPE_CHECKING, Callable, List, Optional, Type, Union

from litestar import Request
from litestar.middleware.base import AbstractMiddleware

if TYPE_CHECKING:
    from litestar.types import ASGIApp, Message, Receive, Scope, Scopes, Send

EmitCallable = Callable[[Request, float], None]


def create_timing_middleware(
    emit: EmitCallable,
    exclude: Optional[Union[str, List[str]]] = None,
    exclude_opt_key: Optional[str] = "exclude_timing",
    scopes: Optional["Scopes"] = None,
) -> Type[AbstractMiddleware]:
    """
    Basic function to create a timing middleware.

    emit: A Callable that accept two arguments, Request and float.

    exclude: A string or a list of strings of paths that are omitted.

    exclude_opt_key: A key to omit the middleware from a specific route.
    Default `exclude_timing`
    scopes:  Can be set to `ScopeType.HTTP` and `ScopeType.WEBSOCKET`.
    The default is both.

    For more informations about the fields check the Litestar docs [Creating Middleware](https://docs.litestar.dev/2/usage/middleware/creating-middleware.html)
    """

    class BaseTimingMiddleware(AbstractMiddleware):
        def __init__(self, app: "ASGIApp") -> None:
            super().__init__(app, exclude, exclude_opt_key, scopes)

        async def __call__(
            self, scope: "Scope", receive: "Receive", send: "Send"
        ) -> None:
            start_time = time()
            request: Request = Request(scope)

            async def send_wrapper(message: "Message") -> None:
                if message["type"] == "http.response.body":
                    emit(request, time() - start_time)
                await send(message)

            await self.app(
                scope,
                receive,
                send_wrapper,
            )

    return BaseTimingMiddleware
